// COlor.h

// This file defines methods that are useful for manipulating the color of individual locations in a bitmap.


#include <math.h>
#include "bitmap.h"
#include "color.h"

#define PI 3.14159265


void testIt()
{

 int r, g, b;
 for (r = 0; r < 255; r++)
 {
   for (g = 0; g < 255; g++)
   {
	 for (b = 0; b < 255; b++)
	 {
	 b=g;
		struct Pixel p;
 
		p.Green = g;
		p.Red = r;
		p.Blue = b;
 
		convertRGBToHSI(&p, &p.hsirepresentation);
 		convertHSIToRGB(&p.hsirepresentation, &p);


		if (p.Green != g || p.Red != r || p.Blue != b)
		{
		  printf("<<<<< Error: Expected %d %d %d Actual: %d %d %d %f %f %f\n", r, g, b, p.Red, p.Green, p.Blue , p.hsirepresentation.hue, p.hsirepresentation.saturation,p.hsirepresentation.intensity);
		}
		}
		}
		}
	
	
}

/**********************************************************************************************
 * This method will convert a pixel from RGB to HSI representation.
 *
 * @param  struct Pixel* src - This is the source pixel that has a representation in RGB format that is to be converted.
 * @param  struct hsiPixel* destination - This is the destination pixel that is to be written in hsi format.
 * @return There is no return from this method.
 **********************************************************************************************/
void convertRGBToHSI(struct Pixel* src, struct hsiPixel* destination)
{
  double r;
  double g;
  double b;

  r = src->Red / 255.0;
  g = src->Green / 255.0;
  b = src ->Blue / 255.0;
  
  destination->intensity = (r+b+g)/3.0;
  
  double min = r;
  if (g < min)
  {
    min = g;
  }
  if (b < min)
  {
    min = b;
  }
  
  destination->saturation = 1.0 - (3.0 / (r + b + g)) * (double)min;
  double p1 = .5 * (r - g + r - b);
  double p2 = (r - g)*(r - g) + (r - b)*(g - b);
  double p3 = sqrt(p2);
  destination -> hue = acos(p1 / p3) * 180 / PI;
  
  if (b > g)
  {
    destination->hue = 360.0 - destination->hue;
  }
  
}

/**********************************************************************************************
 * This method will convert a pixel from RGB to HSI representation.
 *
 * @param  struct hsiPixel* src - This is the source pixel that is to be written in hsi format.
 * @param  struct Pixel* destination - This is the destination pixel that has a representation in RGB format that is to be converted.
 * @return There is no return from this method.
 **********************************************************************************************/
void convertHSIToRGB(struct hsiPixel *src, struct Pixel* destination)
{
	double h;
	double b;
	double r;
	double g;
	
  if (src->saturation==0)
  {
    // Pure white.
	r = 1.0 / 3;
	g = 1.0 / 3;
	b = 1.0 / 3;
  }
  else if ((0.0 <= src->hue) && (src->hue <= 120))
  {
	 h = src->hue;
	 b = (1-src->saturation)/3;
	 r = (1+(src->saturation * cos(h* PI / 180.0) / cos((60 - h) * PI / 180.0) ))/3.0;
	 g = 1 - (r+b);
  }
  else if ((120 <= src->hue) && (src->hue <= 240))
  {
     h = src->hue - 120;
	 g = (1+((src->saturation * cos(h* PI / 180.0)) / cos((60 - h) * PI / 180.0) ))/3.0;
     r = (1-src->saturation)/3;
	 b = 1 - (r+g);
  }
  else if ((240 <= src->hue) && (src->hue <= 360))
  {
     h = src->hue - 240;
	 b = (1+(src->saturation * cos(h* PI / 180.0) / cos((60 - h) * PI / 180.0) ))/3.0;
     g = (1-src->saturation)/3;
	 r = 1 - (g+b);
  }
  
  r = 3 * src->intensity * r;
  g = 3 * src->intensity * g;
  b = 3 * src->intensity * b;
  
  destination->Red =   ((char)(r * 255));
  destination->Green = ((char)(g*255));
  destination->Blue =  ((char)(b * 255));
} 