#include <math.h>
#include "bitmap.h"
#include "contrastStretch.h"
#include "color.h"

/**********************************************************************************************
 * This method will stretch the contrast of a given image.  It first converts the image to HSI format,
 * then adjusts the convert a pixel from RGB to HSI representation.  The intensity will then be 
 * stretched over the range provided, and the image will then be converted back to RGB coordinates.
 *
 * @param  struct BitMap *image This is a pointer to the image that is to be converted.
 * @param double desigedMin - This is the minimum intensity that should be in the image.
 * @param double desiredMax - This si the maximum intensity that should be in the given image.
 * @return There is no return from this method.
 **********************************************************************************************/
void stretchContrast(struct BitMap *image, double desiredMin, double desiredMax)
{
    int i, j;
	
	double minIntensity=1;
	double maxIntensity=-1;
	
	// Start by converting the image to HSI and trying to find the max and min values for the given image.
    for(i=0; i < image->height; i++) {
      for(j=0; j < image->width; j++) {
		// Convert the pixel to hsi.
   	    struct Pixel *cp = getPixel(image, j, i);
		convertRGBToHSI(cp,&cp->hsirepresentation);
		
		// Now see if the pixel is greater than the max or less than the min.
		if (cp->hsirepresentation.intensity>maxIntensity)
		{
		  maxIntensity = cp->hsirepresentation.intensity;
		}
		if (cp->hsirepresentation.intensity<minIntensity)
		{
		  minIntensity = cp->hsirepresentation.intensity;
		}
      }
	} 

    // Now that we have max and min values, do the scaling and convert each pixel back to RGB.
	double divisor = maxIntensity - minIntensity;
	
    for(i=0; i < image->height; i++) {
      for(j=0; j < image->width; j++) {
		// Scale the pixel.
   	    struct Pixel *cp = getPixel(image, j, i);
		double temp;
		// Scale it first over the range 0 to 1.
		temp = (cp->hsirepresentation.intensity - minIntensity) / divisor;
		
		// Now adjust to the range passed in.
		temp = temp * (desiredMax - desiredMin);
		
		//Now add the offset and store.
		cp->hsirepresentation.intensity = temp + desiredMin;
		
		// Now convert the pixel back to RGB.
		convertHSIToRGB(&cp->hsirepresentation, cp);
      }
	} 
}