// convolutions.h
#ifndef _CONVOLUTIONS_H
#define _CONVOLUTIONS_H
#include "bitmap.h"

/**********************************************************************************************
 * This method will perform a convolution on a 3 x 3 pixel matrix.  Essentially, the value 
 * provided by each pixel will be multiplied by a given factor and the result placed in the 
 * given pixel.
 *
 * @param struct Pixel* pixels This is a pointer to an array of 9 pixels which are to be involved
 *        in the convolution.  The pixels are UL, T, UR, L, Middle, R, LL, B, LR on positions 0 - 8.
 * @param int *coeffecients This is the coefficient as to how much of the value is to be kept in the final convolution.
 * @param int* retValBitMap This is a pointer to the R, G, and B values that are the result of this convolution.
 * @return There is no return from this method.
 **********************************************************************************************/
void convolve3x3(struct Pixel* pixels, int *coeffecients, int* retVal);

/**********************************************************************************************
 * This method will perform a convolution on a 3 x 3 pixel matrix.  Essentially, the value 
 * provided by each pixel will be multiplied by a given factor and the result placed in the 
 * given pixel.  The type specifies what aspect (H, S, or I) should be used for the convolve.
 *
 * @param struct Pixel* pixels This is a pointer to an array of 9 pixels which are to be involved
 *        in the convolution.  The pixels are UL, T, UR, L, Middle, R, LL, B, LR on positions 0 - 8.
 * @param int *coeffecients This is the coefficient as to how much of the value is to be kept in the final convolution.
 * @param int* type This is the type of the convolve.  This should be either HUE, SATURATION, or INTENSITY.
 * @return The result of the convolution will be returned.
 **********************************************************************************************/
#define HUE (0x01)
#define SATURATION (0x02)
#define INTENSITY (0x04)
double hsiConvolve3x3(struct hsiPixel* pixels, int *coeffecients, int type);

#endif
