#include <math.h>
#include "bitmap.h"
#include "gammaCorrector.h"
#include "color.h"


/**********************************************************************************************
 * This method will adjust the gamma of the image.  The gamma impacts the relationship between
 * the brightness and darkness of a pixel.  This is done in the HSI image space.
 *
 * @param  double gamma - This is the gamma that is to be applied to the image.
 * @param  struct BitMap *image This is the image that is to be manipulated.
 * @return There is no return from the main method.
 **********************************************************************************************/
void correctGamma(double gamma, struct BitMap *image)
{
	int i;
	int j;
	// Start by converting the image to HSI and trying to find the max and min values for the given image.
    for(i=0; i < image->height; i++) {
      for(j=0; j < image->width; j++) {
		// Convert the pixel to hsi.
   	    struct Pixel *cp = getPixel(image, j, i);
		convertRGBToHSI(cp,&cp->hsirepresentation);
		
		// Now that we are in HSI, apply the gamma function to the pixel intensity.
		cp->hsirepresentation.intensity = pow(cp->hsirepresentation.intensity, 1.0/gamma);
		
		// Now convert the pixel back to RGB.
		convertHSIToRGB(&cp->hsirepresentation, cp);
      }
	} 
}